#!/usr/bin/env python2
# -*- coding: UTF-8 -*-

import ConfigParser
import os
import re
import subprocess
from _gmutils.option import Option
from graphicsmode import XRandr

notAssigned = 'Не назначено'
manual = 'Вручную'
isPosSco = False

# Проверяем используется ли касса-трансформер
process = subprocess.Popen(['apt', 'list', '--installed', 'artix-pos-sco'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
stdout, stderr = process.communicate()
stdout = stdout.strip()
if len(stdout.split('\n')) > 1:
    isPosSco = True

class Config(object):
    """
    Предоставляет настройки из ini файла
    """
    def __init__(self, readOnly=False):
        self.iniFile = "/linuxcash/cash/conf/graphicsmode.ini"
        self.config = ConfigParser.RawConfigParser()

        if readOnly:
            self.config.read(self.iniFile)
        else:
            # открытие конфига
            if os.path.isfile(self.iniFile):
                self.config.read(self.iniFile)
                self.checkMainSection()
            else:
                self.checkMainSection()
                self.write()

    def getBool(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.getboolean(section, option)
        except:
            pass
        return value

    def getString(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.get(section, option)
        except:
            pass
        return value

    def getStringList(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = list(filter(lambda x: x, self.config.get(section, option).split(';')))
        except:
            pass
        return value

    def getOption(self, option):
        try:
            value = self.config.get(option.section, option.name)
        except:
            value = option.defaultValue
        return value

    def sections(self):
        return self.config.sections()

    def set(self, section, option, value):
        self.config.set(section, option, value)

    def setOption(self, option, value):
        if not value:
            self.config.remove_option(option.section, option.name)
        else:
            self.config.set(option.section, option.name, value)

    def checkMainSection(self):
        """
        Функция для проверки главной секции в конфиге
        """
        if not self.config.has_section('main'):
            self.config.add_section('main')
        if self.getBool('main', 'setMode', False):
            self.config.set('main', 'setMode', 'True')
        else:
            self.config.set('main', 'setMode', 'False')

    def checkSection(self, section):
        """
        Функция для проверки секции в конфиге
        """
        if not self.config.has_section(section):
            self.config.add_section(section)

    def removeSection(self, section):
        self.config.remove_section(section)

    def setScreenConfiguration(self, screenConfig, primary):
        if screenConfig.initSection != notAssigned:
            self.removeSection(screenConfig.initSection)
            
        section = screenConfig.interfaceOption.value
        screenConfig.initSection = section
        if section == notAssigned:
            return

        self.checkSection(section)
        
        for option in screenConfig.getOptions(primary=primary, forced=True):
            if option.value and option.value != notAssigned and option.section != notAssigned:
                self.setOption(option, option.value)

    def removeScreenSections(self):
        """
        Удаление всех секций с настройками экранов
        """
        sections = self.sections()
        sections.remove('main')
        for section in sections:
            self.removeSection(section)

    def removeLastScreenSection(self):
        """
        Удаление секции настроек второго экрана
        """
        sections = self.sections()
        sections.remove('main')
        if len(sections) == 2:
            self.removeSection(sections[1])

    def getUsedInterfaces(self):
        """
        Список используемых интерфейсов
        """
        return [screen.interfaceOption.value for screen in self.getScreenConfigurations()]

    def getScreenConfigurations(self):
        sections = self.sections()
        sections.remove('main')
        result = list()

        screens = 0
        primaryFound = False
        for screenSection in sections:
            if screens == 2:
                break
            screenConfiguration = ScreenConfiguration(screenSection)
            screenConfiguration.interfaceOption.value = screenSection

            for option in screenConfiguration.getOptions():
                option.value = self.getOption(option)

            result.append(screenConfiguration)
            primaryFound = (screenConfiguration.primary.value == 'True')
            screens = screens + 1

        # миграция настроек
        if screens and not self.getString('main', 'screens', ''):
            self.set('main', 'screens', 'one')

        # есть несколько ситуаций:
        # 1. primary указан в настройках, но они ещё не применились - это и будет primary-монитор
        # 2. primary не указан в настройках, однако уже есть primary-монитор - это и будет primary-монитор
        # 3. primary не указан в настройках и нет primary-монитора. Тогда полагаем, что первый в списке от xrandr и будет primary
        if not primaryFound:
            output = XRandr().getPrimaryOutput()
            for screenConfiguration in result:
                if screenConfiguration.interfaceOption.value == output:
                    screenConfiguration.setPrimary('True')
                else:
                    screenConfiguration.setPrimary('False')
        return result

    def write(self):
        """
        Запись конфигурации в файл
        """
        with open(self.iniFile, 'wb') as configFile:
            self.config.write(configFile)

    def hasOption(self, section, option):
        return self.config.has_option(section, option)

    def removeOption(self, section, option):
        return self.config.remove_option(section, option)


class ScreenConfiguration(object):
    boolItems = {'True': 'Да', 'False': 'Нет'}
    """
    Настройки экрана
    """

    def __init__(self, section):
        self.rotates = {'normal': 'Отсутствует', 'left': 'Влево', 'right': 'Вправо', 'inverted': 'Инвертировано'}
        self.reflects = {'normal': 'Отсутствует', 'x': 'Горизонтально', 'y': 'Вертикально',
                         'xy': 'Горизонтально и Вертикально'}

        self.modeOption = Option('mode', 'listWithEntry', section, 'Разрешение экрана', 'Разрешение экрана', '', [manual])
        # функция получения списка разрешений для экрана
        # берёт список разрешений и объединяет его с настройкой, если она представляет собой корректное разрешение,
        # котрого нет в списке, затем сортирует список в порядке убывания и добавляет элемент "Вручную" в конец списка
        self.modeOption.paramsFunc = lambda params, value: sorted(
                set(params) | (set([value] if re.search(r'^[0-9]+x[0-9]+$', value) else []))
                , key=lambda s: int(s.split('x')[0]), reverse=True) + [manual]
        self.modeOption.checkValueFunc = lambda value: re.search(r'^[0-9]+x[0-9]+$', value) is not None

        self.interfaceOption = Option('interface', 'list', section, 'Интерфейс',
                                      'Выбор интерфейса, к которому подключен экран', notAssigned, list())
        self.scaleOption = Option('scale', 'entry', section, 'Изменение размера',
                                  'Изменение размера выводимого изображения.\nПо умолчанию 1.0x1.0', '1.0x1.0')
        self.reflectOption = Option('reflect', 'dict', section, 'Отражение', 'Отражение по вертикали или горизонтали',
                                    'normal', self.reflects)
        self.rotateOption = Option('rotate', 'dict', section, 'Поворот экрана', 'Настройка поворота экрана',
                                   'normal', self.rotates)
        self.gammaOption = Option('gamma', 'entry', section, 'Цветовая гамма',
                                  'Установка цветовой гаммы.\nПо умолчанию 1.0:1.0:1.0', '1.0:1.0:1.0')

        self.duplicate = Option('duplicate', 'dict', section, 'Дублирование', 'Дублирование основного экрана', 'False',
                                self.boolItems)

        self.inputOption = Option('inputs', 'checkboxtree', section, 'Маппинг ввода',
                                  'Маппинг устройств ввода на соответствующий экран', '')

        self.application = Option('application', 'listWithEntry', section, 'Приложение', 'Приложение для запуска',
                                  notAssigned, [notAssigned, '/linuxcash/cash/view/artix-view', '/usr/bin/cutecom', manual])
        # функция получения списка приложений
        # если приложения нет в списке, то осуществляет вставку в предпоследнюю позицию
        self.application.paramsFunc = lambda params, value: (params if value in params
                else params[:-1] + [value] + params[-1:])

        self.runAppWith = Option('runAppWith', 'list', section, 'Тип кассы',
                                 'Запускать приложение только вместе с указанным типом кассы', notAssigned,
                                 [notAssigned, 'pos', 'sst-sco'])

        self.options = [self.modeOption, self.scaleOption, self.reflectOption, self.rotateOption, self.gammaOption,
                        self.duplicate, self.application, self.runAppWith, self.inputOption]

        # при смене интерфейса экрана необходимо удалить секцию, к которой настройки относились изначально
        self.initSection = section

        self.primary = Option('primary', 'str', section, hidden=True)

    def getOptions(self, primary=False, forced=False):
        """
        Получения списка опций экрана
        :param forced получение всех опций для записи в файл
        """
        ignored = []
        if primary:
            self.primary.value = 'True'
            self.duplicate.hidden = True
            self.application.hidden = True
            self.runAppWith.hidden = True
            ignored.append(self.duplicate)
            ignored.append(self.runAppWith)
        else:
            self.primary.value = 'False'
            self.duplicate.hidden = False
            if self.duplicate.value == 'True':
                self.application.hidden = True
                ignored.append(self.application)
            else:
                self.application.hidden = False
            self.runAppWith.hidden = self.application.hidden
            if not isPosSco or self.runAppWith.hidden:
                ignored.append(self.runAppWith)

        result = []
        if forced:
            result.append(self.primary)

        for option in self.options:
            if (not option.hidden or forced) and (option not in ignored):
                result.append(option)

        return result

    def setModes(self, modes):
        """
        Установка доступных для экрана режимов
        """
        self.modeOption.params = modes

    def setInterfaces(self, interfaces):
        """
        Установка доступных для настройки интерфейсов
        """
        self.interfaceOption.params = interfaces

    def setSection(self, section):
        """
        Установка секции для настроек экрана
        """
        for option in self.getOptions(forced=True):
            option.section = section

    def getAnnotation(self):
        """
        Строка для отображения в списке настроек
        """
        if self.interfaceOption.value == notAssigned:
            return "Не сконфигурирован"
        return "%-6s %s" % (self.interfaceOption.value, self.modeOption.value)

    def setPrimary(self, value):
        self.primary.value = value
